# -*- coding: utf-8 -*-
"""
Created on Thu Mar  5 10:43:56 2026
@author: Nicolas Tharaud
          __  
    ___  |_|____ ___
  / __ \/ / ___/ __ \
 / / / / / |__| |_| |
/_/ /_/_/\___/\____/T
"""

from matplotlib.colors import LinearSegmentedColormap
from .loader import load_hex
import importlib.resources as pkg_resources
import numpy as np
import matplotlib.pyplot as plt

# --- Palette as a list of HEX colors
def pal_hex(name):
    return load_hex(name)

# --- Palette as a matplotlib colormap
def pal(name, N=256, reverse=False):
    colors = load_hex(name)
    if reverse:
        colors = colors[::-1]
    return LinearSegmentedColormap.from_list(name, colors, N=N)

# --- List of Nicopal available
def pal_list():
    files = pkg_resources.contents("nicopal.data")
    palettes = set()
    for f in files:
        if f.endswith("_hex.json"):
            name = f.split("_")[1]   
            palettes.add(name)
    return sorted(list(palettes))

# --- Visualize a palette
def pal_show(name, N=256):
    colmap = pal(name, N)
    gradient = np.linspace(0,1,N).reshape(1,-1)
    plt.figure(figsize=(6,1))
    plt.imshow(gradient, aspect="auto", cmap=colmap)
    plt.axis("off")
    plt.title(name)
    plt.show()
    
# --- Visualize all palettes
def pal_gallery(N=256, ncols=4, reverse=False):
    palettes = pal_list()
    n = len(palettes)
    nrows = int(np.ceil(n / ncols))
    fig, axes = plt.subplots(nrows, ncols, figsize=(3*ncols, 1.2*nrows))
    gradient = np.linspace(0, 1, N).reshape(1, -1)
    axes = np.array(axes).reshape(-1)
    for ax, p in zip(axes, palettes):
        colmap = pal(p, N=N, reverse=reverse)
        ax.imshow(gradient, aspect="auto", cmap=colmap)
        ax.set_axis_off()
        ax.set_title(p, fontsize=10)
    for ax in axes[len(palettes):]:
        ax.axis("off")
    plt.tight_layout()
    plt.show()
    
def pal_all():
    pal_gallery()
    
    
# --- Palette valid
def pal_test():
    for p in pal_list():
        assert len(pal_hex(p)) > 0, f"Palette {p} empty"
    print("All palettes are valid!")
    
# --- Return n discrete colors
def pal_sample(name, n, reverse=False):
    colmap = pal(name, N=n, reverse=reverse)
    return [colmap(i/(n-1)) for i in range(n)]

# --- Show a demo plot
def pal_demo(name, size=30):
    data = np.random.rand(size, size)
    plt.figure(figsize=(5,4))
    im = plt.imshow(data, cmap=pal(name))
    plt.colorbar(im)
    plt.title(f"Nicopal demo: {name}")
    plt.axis("off")
    plt.show()


# -*- coding: utf-8 -*-
"""
Created on Thu Mar  5 10:34:46 2026
@author: Nicolas Tharaud
          __  
    ___  |_|____ ___
  / __ \/ / ___/ __ \
 / / / / / |__| |_| |
/_/ /_/_/\___/\____/T
"""

import json
import pickle
import importlib.resources as pkg_resources
from .exceptions import PaletteNotFoundError

def find_file(palette_name, suffix):
    files = pkg_resources.contents("nicopal.data")
    for f in files:
        if palette_name in f and f.endswith(suffix):
            return f
    raise PaletteNotFoundError(f"Palette {palette_name} not found")


def load_hex(name):
    filename = find_file(name, "_hex.json")
    with pkg_resources.open_text("nicopal.data", filename) as f:
        return json.load(f)


def load_rgb(name):
    filename = find_file(name, "_rgb.json")
    with pkg_resources.open_text("nicopal.data", filename) as f:
        return json.load(f)

_CACHE = {}

def load_cmap(name):
    if name in _CACHE:
        return _CACHE[name]
    filename = find_file(name, "_cmap.pkl")
    with pkg_resources.open_binary("nicopal.data", filename) as f:
        cmap = pickle.load(f)
    _CACHE[name] = cmap
    return cmap

    
    


# -*- coding: utf-8 -*-
"""
Created on Wed Mar 11 01:46:20 2026
@author: Nicolas Tharaud
          __  
    ___  |_|____ ___
  / __ \/ / ___/ __ \
 / / / / / |__| |_| |
/_/ /_/_/\___/\____/T
"""

"""
Nicopal Version Information
"""

__version__ = """\
Nicopal - version 0.3.1
Author : Nicolas Tharaud <n.tharaud6@gmail.com>
Copyright (c) 2026 Nicolas Tharaud
Colour palettes designed by a protanopic student
"""

def show_info():
    print(__version__)

if __name__ == "__main__":
    show_info()


# -*- coding: utf-8 -*-
"""
Created on Wed Mar 11 01:56:39 2026
@author: Nicolas Tharaud
          __  
    ___  |_|____ ___
  / __ \/ / ___/ __ \
 / / / / / |__| |_| |
/_/ /_/_/\___/\____/T
"""

class PaletteNotFoundError(Exception):
    """- Invalid Nicopal palette name"""
    pass


# -*- coding: utf-8 -*-
"""
Created on Thu Mar  5 10:46:13 2026
@author: Nicolas Tharaud
          __  
    ___  |_|____ ___
  / __ \/ / ___/ __ \
 / / / / / |__| |_| |
/_/ /_/_/\___/\____/T
"""

from .__version__ import __version__ 
from .palettes import pal_hex, pal, pal_list, pal_show # v0.1.0
from .palettes import pal_test # v0.3.0
from .palettes import pal_gallery, pal_all, pal_sample, pal_demo # v0.3.1